(* This file extracted from the GTK tutorial.
 *
 * notebook.c
 *  
 * Converted from C to Pascal by Frank Loemker
 * <floemker@techfak.uni-bielefeld.de>
 *)

uses glib, gtkmain, gtkwidget, gtkbutton, gtkwindow, gtkenums, gtksignal,
     gtkobjects, gtkcontainer,
     gtklabel, gtktable, gtknotebook, gdktypes, gtkframe, gtkcheckbutton;

Function itos (I : Longint) : String;
Var S : String[15];
begin
  Str (I,S);
  itos:=S;
end;

(* This function rotates the position of the tabs *)
procedure rotate_book (button : pGtkButton ; notebook:pGtkNotebook ); cdecl;
begin
  gtk_notebook_set_tab_pos (notebook, (tab_pos(notebook^) +1) mod 4);
end;

(* Add/Remove the page tabs and the borders *)
procedure tabsborder_book (button : pGtkButton ; notebook:pGtkNotebook ); cdecl;
var tval, bval : gint;
begin
  tval := 0;
  bval := 0;
  if show_tabs(notebook^) = 0 then
    tval := 1;
  if show_border(notebook^) = 0 then
    bval := 1;

  gtk_notebook_set_show_tabs (notebook, tval);
  gtk_notebook_set_show_border (notebook, bval);
end;

(* Remove a page from the notebook *)
procedure remove_book (button : pGtkButton ; notebook: pGtkNotebook ); cdecl;
var page : gint ;
begin
  page := gtk_notebook_current_page(notebook);
  gtk_notebook_remove_page (notebook, page);
  (* Need to refresh the widget --
   This forces the widget to redraw itself. *)
  gtk_widget_draw(pGTKWIDGET(notebook), NIL);
end;

procedure delete (widget : pGtkWidget ; event: pGdkEvent; data: pgpointer ); cdecl;
begin
  gtk_main_quit ();
end;

var window, button, table, notebook, frame, thelabel, checkbutton : pGtkWidget;
  i								  : integer;
  bufferf, bufferl						  : string[33];
begin
  gtk_init (@argc, @argv);

  window := gtk_window_new (GTK_WINDOW_TOPLEVEL);

  gtk_signal_connect (pGTKOBJECT (window), 'delete_event',
		      GTKSIGNALFUNC (@delete), NIL);

  gtk_container_border_width (pGTKCONTAINER (window), 10);

  table := gtk_table_new(2,6,FALSE);
  gtk_container_add (pGTKCONTAINER (window), table);

  (* Create a new notebook, place the position of the tabs *)
  notebook := gtk_notebook_new ();
  gtk_notebook_set_tab_pos (pGTKNOTEBOOK (notebook), GTK_POS_TOP);
  gtk_table_attach_defaults(pGTKTABLE(table), notebook, 0,6,0,1);
  gtk_widget_show(notebook);

  (* lets append a bunch of pages to the notebook *)
  for i:=0 to 4 do begin
    bufferf := 'Append Frame '+itos(i+1)+#0;
    bufferl := 'Page '+itos(i+1)+#0;

    frame := gtk_frame_new (pchar(@bufferf[1]));
    gtk_container_border_width (pGTKCONTAINER (frame), 10);
    gtk_widget_set_usize (frame, 100, 75);
    gtk_widget_show (frame);

    thelabel := gtk_label_new (pchar(@bufferf[1]));
    gtk_container_add (pGTKCONTAINER (frame), thelabel);
    gtk_widget_show (thelabel);

    thelabel := gtk_label_new (pchar(@bufferl[1]));
    gtk_notebook_append_page (pGTKNOTEBOOK (notebook), frame, thelabel);
  end;


  (* now lets add a page to a specific spot *)
  checkbutton := gtk_check_button_new_with_label ('Check me please!');
  gtk_widget_set_usize(checkbutton, 100, 75);
  gtk_widget_show (checkbutton);

  thelabel := gtk_label_new ('Add spot');
  gtk_container_add (pGTKCONTAINER (checkbutton), thelabel);
  gtk_widget_show (thelabel);
  thelabel := gtk_label_new ('Add page');
  gtk_notebook_insert_page (pGTKNOTEBOOK (notebook), checkbutton, thelabel, 2);

  (* Now finally lets prepend pages to the notebook *)
  for i:=0 to 4 do begin
    bufferf := 'Prepend Frame '+itos(i+1)+#0;
    bufferl := 'PPage '+itos(i+1)+#0;

    frame := gtk_frame_new (pchar(@bufferf[1]));
    gtk_container_border_width (pGTKCONTAINER (frame), 10);
    gtk_widget_set_usize (frame, 100, 75);
    gtk_widget_show (frame);

    thelabel := gtk_label_new (pchar(@bufferf[1]));
    gtk_container_add (pGTKCONTAINER (frame), thelabel);
    gtk_widget_show (thelabel);

    thelabel := gtk_label_new (pchar(@bufferl[1]));
    gtk_notebook_prepend_page (pGTKNOTEBOOK(notebook), frame, thelabel);
  end;

  (* Set what page to start at (page 4) *)
  gtk_notebook_set_page (pGTKNOTEBOOK(notebook), 3);


  (* create a bunch of buttons *)
  button := gtk_button_new_with_label ('close');
  gtk_signal_connect_object (pGTKOBJECT (button), 'clicked',
			     GTKSIGNALFUNC (@delete), NIL);
  gtk_table_attach(pGTKTABLE(table), button, 0,1,1,2,
		   GTK_FILL or GTK_EXPAND,GTK_FILL,0,0);
  gtk_widget_show(button);

  button := gtk_button_new_with_label ('next page');
  gtk_signal_connect_object (pGTKOBJECT (button), 'clicked',
			     GtkSignalFunc (@gtk_notebook_next_page),
			     pGTKOBJECT (notebook));
  gtk_table_attach(pGTKTABLE(table), button, 1,2,1,2,
		   GTK_FILL or GTK_EXPAND,GTK_FILL,0,0);
  gtk_widget_show(button);

  button := gtk_button_new_with_label ('prev page');
  gtk_signal_connect_object (pGTKOBJECT (button), 'clicked',
			     GtkSignalFunc (@gtk_notebook_prev_page),
			     pGTKOBJECT (notebook));
  gtk_table_attach(pGTKTABLE(table), button, 2,3,1,2,
		   GTK_FILL or GTK_EXPAND,GTK_FILL,0,0);
  gtk_widget_show(button);

  button := gtk_button_new_with_label ('tab position');
  gtk_signal_connect_object (pGTKOBJECT (button), 'clicked',
			     GtkSignalFunc (@rotate_book), pGTKOBJECT(notebook));
  gtk_table_attach(pGTKTABLE(table), button, 3,4,1,2,
		   GTK_FILL or GTK_EXPAND,GTK_FILL,0,0);
  gtk_widget_show(button);

  button := gtk_button_new_with_label ('tabs/border on/off');
  gtk_signal_connect_object (pGTKOBJECT (button), 'clicked',
			     GtkSignalFunc (@tabsborder_book),
			     pGTKOBJECT (notebook));
  gtk_table_attach(pGTKTABLE(table), button, 4,5,1,2,
		   GTK_FILL or GTK_EXPAND,GTK_FILL,0,0);
  gtk_widget_show(button);

  button := gtk_button_new_with_label ('remove page');
  gtk_signal_connect_object (pGTKOBJECT (button), 'clicked',
			     GtkSignalFunc (@remove_book),
			     pGTKOBJECT(notebook));
  gtk_table_attach(pGTKTABLE(table), button, 5,6,1,2,
		   GTK_FILL ,GTK_FILL,0,0);
  gtk_widget_show(button);

  gtk_widget_show(table);
  gtk_widget_show(window);

  gtk_main ();
end.
