(* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Converted from C to Pascal by Frank Loemker
 * <floemker@techfak.uni-bielefeld.de>
 *)

uses glib, gtkmain, gtkwidget, gtkbutton, gtkwindow, gtkenums, gtksignal,
     gtkobjects, gtkbox, gtkcontainer, gdktypes, gdk, gtkvbox, gtkdrawingarea;

(* Backing pixmap for drawing area *)
const pixmap : PGdkPixmap = NIL;

(* Create a new backing pixmap of the appropriate size *)
function configure_event (widget : pGtkWidget; event: pGdkEventConfigure):gint; cdecl;
begin
  if pixmap<>NIL then
    gdk_pixmap_unref(pixmap);

  pixmap := gdk_pixmap_new(widget^.window,
			   widget^.allocation.width,
			   widget^.allocation.height,
			   -1);
  gdk_draw_rectangle (pixmap,
		      widget^.thestyle^.white_gc,
		      TRUE,
		      0, 0,
		      widget^.allocation.width,
		      widget^.allocation.height);

  configure_event := TRUE;
end;

(* Redraw the screen from the backing pixmap *)
function expose_event (widget : pGtkWidget ; event : pGdkEventExpose ) : gint; cdecl;
begin
  gdk_draw_pixmap(widget^.window,
		  widget^.thestyle^.fg_gc[GTK_WIDGET_STATE (widget)],
		  pixmap,
		  event^.area.x, event^.area.y,
		  event^.area.x, event^.area.y,
		  event^.area.width, event^.area.height);
  expose_event:= FALSE;
end;

(* Draw a rectangle on the screen *)
procedure draw_brush (widget : pGtkWidget ; x, y: gint16);
var update_rect	: GdkRectangle;
begin
  update_rect.x := x - 5;
  update_rect.y := y - 5;
  update_rect.width := 10;
  update_rect.height := 10;
  gdk_draw_rectangle (pixmap,
		      widget^.thestyle^.black_gc,
		      TRUE,
		      update_rect.x, update_rect.y,
		      update_rect.width, update_rect.height);
  gtk_widget_draw (widget, @update_rect);
end;

function button_press_event (widget : pGtkWidget ; event: pGdkEventButton ) : gint; cdecl;
begin
  if (event^.button = 1) and (pixmap <> NIL) then begin
    draw_brush (widget, trunc(event^.x), trunc(event^.y));
  end;
  button_press_event := TRUE;
end;

function motion_notify_event (widget : pGtkWidget ; event: pGdkEventMotion ) : gint; cdecl;
var x, y : longint ;
  state	 : GdkModifierType;
begin
  if (event^.is_hint<>0) then begin
    gdk_window_get_pointer (event^.window, @x, @y, @state);
  end else begin
    x := trunc(event^.x);
    y := trunc(event^.y);
    state := event^.state;
  end;

  if ((state and GDK_BUTTON1_MASK)<>0) and (pixmap <> NIL) then
    draw_brush (widget, x, y);

  motion_notify_event := TRUE;
end;

procedure quit;
begin
  gtk_exit (0);
end;

var window, drawing_area, vbox, button : pGtkWidget;
begin
  gtk_init (@argc, @argv);

  window := gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (window, 'Test Input');

  vbox := gtk_vbox_new (FALSE, 0);
  gtk_container_add (pGTKCONTAINER (window), vbox);
  gtk_widget_show (vbox);

  gtk_signal_connect (pGTKOBJECT (window), 'destroy',
		      GTKSIGNALFUNC (@quit), NIL);

  (* Create the drawing area *)

  drawing_area := gtk_drawing_area_new ();
  gtk_drawing_area_size (pGTKDRAWINGAREA (drawing_area), 200, 200);
  gtk_box_pack_start (pGTKBOX (vbox), drawing_area, TRUE, TRUE, 0);

  gtk_widget_show (drawing_area);

  (* Signals used to handle backing pixmap *)

  gtk_signal_connect (pGTKOBJECT (drawing_area), 'expose_event',
		      GtkSignalFunc (@expose_event), NIL);
  gtk_signal_connect (pGTKOBJECT(drawing_area),'configure_event',
		      GtkSignalFunc (@configure_event), NIL);

  (* Event signals *)

  gtk_signal_connect (pGTKOBJECT (drawing_area), 'motion_notify_event',
		      GtkSignalFunc (@motion_notify_event), NIL);
  gtk_signal_connect (pGTKOBJECT (drawing_area), 'button_press_event',
		      GtkSignalFunc (@button_press_event), NIL);

  gtk_widget_set_events (drawing_area, GDK_EXPOSURE_MASK
			 or GDK_LEAVE_NOTIFY_MASK
			 or GDK_BUTTON_PRESS_MASK
			 or GDK_POINTER_MOTION_MASK
			 or GDK_POINTER_MOTION_HINT_MASK);

  (* .. And a quit button *)
  button := gtk_button_new_with_label ('Quit');
  gtk_box_pack_start (pGTKBOX (vbox), button, FALSE, FALSE, 0);

  gtk_signal_connect_object (pGTKOBJECT (button), 'clicked',
			     GTKSIGNALFUNC (@gtk_widget_destroy),
			     pGTKOBJECT (window));
  gtk_widget_show (button);

  gtk_widget_show (window);

  gtk_main ();
end.
