unit tictactoe;

(* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Converted from C to Pascal by Frank Loemker
 * <floemker@techfak.uni-bielefeld.de>
 *)

interface

uses glib, gtksignal, gtktable, gtktogglebutton, gtkvbox, gtkwidget;

{
 #define TICTACTOE(obj)          pGTKCHECK_CAST (obj, tictactoe_get_type (), Tictactoe)
 #define TICTACTOE_CLASS(klass)  pGTKCHECK_CLASS_CAST (klass, tictactoe_get_type (), TictactoeClass)
 #define IS_TICTACTOE(obj)       pGTKCHECK_TYPE (obj, tictactoe_get_type ())
 }

type
  PTictactoe	  = ^TTictactoe;
  TTictactoe	  = record
		      vbox    : TGtkVBox ;
		      buttons : array [0..2 , 0..2] of pGtkWidget;
		    end;
  PTictactoeClass = ^TTictactoeClass;
  TTictactoeClass = record
		      parent_class: TGtkVBoxClass ;
		      tictactoe: Procedure (ttt : pTictactoe); cdecl;
		    end;

Function tictactoe_get_type : guint;
Function tictactoe_new : pGtkWidget;
procedure tictactoe_clear (ttt : pTictactoe);

implementation

uses gtkenums, gtkobjects, gtkcontainer;

const ANZ_SIGNAL = 1;
type TTT_Signals = (TICTACTOE_SIGNAL);

const tictactoe_signals: array[TTT_Signals] of guint = (0);

Procedure tictactoe_toggle (widget : pGtkWidget ; ttt: pTictactoe); cdecl;
const rwins: array[0..7,0..2] of integer = 
  ( ( 0, 0, 0 ), ( 1, 1, 1 ), ( 2, 2, 2 ),
   ( 0, 1, 2 ), ( 0, 1, 2 ), ( 0, 1, 2 ),
   ( 0, 1, 2 ), ( 0, 1, 2 ) );
  cwins:array [0..7,0..2] of integer =
  ( ( 0, 1, 2 ), ( 0, 1, 2 ), ( 0, 1, 2 ),
   ( 0, 0, 0 ), ( 1, 1, 1 ), ( 2, 2, 2 ),
   ( 0, 1, 2 ), ( 2, 1, 0 ) );
var i, k	 : integer;
  success, found : boolean;
begin
  for k:=0 to 7 do
  begin
    success := TRUE;
    found := FALSE;

    for i:=0 to 2 do
    begin
      success := success and
      boolean(active(pGTKTOGGLEBUTTON(ttt^.buttons[rwins[k,i],cwins[k,i]])^));
      found := found or
      (ttt^.buttons[rwins[k,i],cwins[k,i]] = widget);
    end;

    if (success and found) then
    begin
      gtk_signal_emit0 (pGTKOBJECT (ttt),
			tictactoe_signals[TICTACTOE_SIGNAL]);
      break;
    end;
  end;
end;

Procedure gtk_signal_default_marshallerT(theobject : pGtkObject;
					 func	   : GtkSignalFunc;
					 func_data : gpointer;
					 args	   : pGtkArg); cdecl;
begin
  gtk_signal_default_marshaller (theobject,func,func_data,args);
end;

Procedure tictactoe_class_init (theclass : pTictactoeClass );
var object_class : pGtkObjectClass ;
begin
  object_class := pGtkObjectClass (theclass);

  tictactoe_signals[TICTACTOE_SIGNAL] :=gtk_signal_new0 ('tictactoe',
		      GTK_RUN_FIRST,
		      object_class^.thetype,
		      @theclass^.tictactoe - theclass,
		      @gtk_signal_default_marshallerT, GTK_TYPE_NONE, 0);

  gtk_object_class_add_signals (object_class, pguint(@tictactoe_signals), ANZ_SIGNAL);

  theclass^.tictactoe := NIL;
end;

Procedure tictactoe_init (ttt : pTictactoe );
var table : pGtkWidget ;
  i,j	  : gint;
begin
  table := gtk_table_new (3, 3, TRUE);
  gtk_container_add (pGTKCONTAINER(ttt), table);
  gtk_widget_show (table);

  for i:=0 to 2 do
    for j:=0 to 2 do
    begin
      ttt^.buttons[i][j] := gtk_toggle_button_new ();
      gtk_table_attach_defaults (pGTKTABLE(table), ttt^.buttons[i][j],
				 i, i+1, j, j+1);
      gtk_signal_connect (pGTKOBJECT (ttt^.buttons[i][j]), 'toggled',
			  GTKSIGNALFUNC (@tictactoe_toggle), ttt);
      gtk_widget_set_usize (ttt^.buttons[i][j], 20, 20);
      gtk_widget_show (ttt^.buttons[i][j]);
    end;
end;

Procedure tictactoe_class_init2 (theclass : gpointer ); cdecl;
begin
  tictactoe_class_init (theclass);
end;

Procedure tictactoe_init2 (ttt : gpointer ); cdecl;
begin
  tictactoe_init (ttt);
end;

Function tictactoe_get_type:guint;
const ttt_type : guint	= 0;
  ttt_info: GtkTypeInfo	= (
			   type_name : 'Tictactoe';
			   object_size : sizeof (TTictactoe);
			   class_size : sizeof (TTictactoeClass);
			   class_init_func : @tictactoe_class_init2;
			   object_init_func : @tictactoe_init2;
			   arg_set_func : NIL;
			   arg_get_func : NIL;
			   );
begin
  if (ttt_type = 0) then
    ttt_type := gtk_type_unique (gtk_vbox_get_type (), @ttt_info);

  tictactoe_get_type:= ttt_type;
end;

Function tictactoe_new:pGtkWidget;
begin
  tictactoe_new:= pGTKWIDGET ( gtk_type_new (tictactoe_get_type ()));
end;

Procedure tictactoe_clear (ttt :  pTictactoe );
var i,j	: integer;
begin
  for i:=0 to 2 do
    for j:=0 to 2 do
    begin
      gtk_signal_handler_block_by_data (pGTKOBJECT(ttt^.buttons[i][j]), ttt);
      gtk_toggle_button_set_state (pGTKTOGGLEBUTTON (ttt^.buttons[i][j]),
				   FALSE);
      gtk_signal_handler_unblock_by_data (pGTKOBJECT(ttt^.buttons[i][j]), ttt);
    end;
end;

begin
end.
