unit gtktable;

  interface

uses gdk,glib,gtkcontainer,gtkwidget;

  Type
     PLongint  = ^Longint;
     PByte     = ^Byte;
     PWord     = ^Word;
     PInteger  = ^Integer;
     PCardinal = ^Cardinal;
     PReal     = ^Real;
     PDouble   = ^Double;

  {
  #define GTK_TABLE(obj)          GTK_CHECK_CAST (obj, gtk_table_get_type (), GtkTable)
  #define GTK_TABLE_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gtk_table_get_type (), GtkTableClass)
  #define GTK_IS_TABLE(obj)       GTK_CHECK_TYPE (obj, gtk_table_get_type ())
  }

  type
     PGtkTable = ^TGtkTable;
     PGtkTableClass = ^TGtkTableClass;
     PGtkTableRowCol = ^TGtkTableRowCol;
     PGtkTableChild = ^TGtkTableChild;
{$PACKRECORDS 4}
     TGtkTable = record
          container : TGtkContainer;
          children : PGList;
          rows : PGtkTableRowCol;
          cols : PGtkTableRowCol;
          nrows : guint16;
          ncols : guint16;
          column_spacing : guint16;
          row_spacing : guint16;
          flag0 : word;
       end;

{$PACKRECORDS 4}
     TGtkTableClass = record
          parent_class : TGtkContainerClass;
       end;

{$PACKRECORDS 4}
     TGtkTableChild = record
          widget : PGtkWidget;
          left_attach : guint16;
          right_attach : guint16;
          top_attach : guint16;
          bottom_attach : guint16;
          xpadding : guint16;
          ypadding : guint16;
          flag0 : word;
       end;

{$PACKRECORDS 2}
     TGtkTableRowCol = record
          requisition : guint16;
          allocation : guint16;
          spacing : guint16;
          flag0 : word;
       end;

  const
     bm_homogeneous = 1;
     bp_homogeneous = 0;
  Function  homogeneous(var a : TGtkTable) : guint; 
  Procedure set_homogeneous(var a : TGtkTable; __homogeneous : guint); 

  const
     bm_xexpand = 1;
     bp_xexpand = 0;
     bm_yexpand = 2;
     bp_yexpand = 1;
     bm_xshrink = 4;
     bp_xshrink = 2;
     bm_yshrink = 8;
     bp_yshrink = 3;
     bm_xfill = 16;
     bp_xfill = 4;
     bm_yfill = 32;
     bp_yfill = 5;
  Function  xexpand(var a : TGtkTableChild) : guint; 
  Procedure set_xexpand(var a : TGtkTableChild; __xexpand : guint);
  Function  yexpand(var a : TGtkTableChild) : guint;
  Procedure set_yexpand(var a : TGtkTableChild; __yexpand : guint);
  Function  xshrink(var a : TGtkTableChild) : guint; 
  Procedure set_xshrink(var a : TGtkTableChild; __xshrink : guint); 
  Function  yshrink(var a : TGtkTableChild) : guint; 
  Procedure set_yshrink(var a : TGtkTableChild; __yshrink : guint); 
  Function  xfill(var a : TGtkTableChild) : guint; 
  Procedure set_xfill(var a : TGtkTableChild; __xfill : guint); 
  Function  yfill(var a : TGtkTableChild) : guint; 
  Procedure set_yfill(var a : TGtkTableChild; __yfill : guint); 

  Const
     bm_need_expand = 1;
     bp_need_expand = 0;
     bm_need_shrink = 2;
     bp_need_shrink = 1;
     bm_expand = 4;
     bp_expand = 2;
     bm_shrink = 8;
     bp_shrink = 3;
  Function  need_expand(var a : TGtkTableRowCol) : guint;
  Procedure set_need_expand(var a : TGtkTableRowCol; __need_expand : guint);
  Function  need_shrink(var a : TGtkTableRowCol) : guint;
  Procedure set_need_shrink(var a : TGtkTableRowCol; __need_shrink : guint);
  Function  expand(var a : TGtkTableRowCol) : guint;
  Procedure set_expand(var a : TGtkTableRowCol; __expand : guint);
  Function  shrink(var a : TGtkTableRowCol) : guint;
  Procedure set_shrink(var a : TGtkTableRowCol; __shrink : guint);

  Function  gtk_table_get_type:guint; cdecl;

  Function  gtk_table_new(rows:gint; columns:gint; homogeneous:gint):PGtkWidget; cdecl;

  Procedure gtk_table_attach(table:pGtkTable; child:pGtkWidget; left_attach:gint; right_attach:gint; top_attach:gint; 
              bottom_attach:gint; xoptions:gint; yoptions:gint; xpadding:gint; ypadding:gint); cdecl;

  Procedure gtk_table_attach_defaults(table:pGtkTable; widget:pGtkWidget; left_attach:gint; right_attach:gint; top_attach:gint; 
              bottom_attach:gint); cdecl;

  Procedure gtk_table_set_row_spacing(table:pGtkTable; row:gint; spacing:gint); cdecl;

  Procedure gtk_table_set_col_spacing(table:pGtkTable; column:gint; spacing:gint); cdecl;

  Procedure gtk_table_set_row_spacings(table:pGtkTable; spacing:gint); cdecl;

  Procedure gtk_table_set_col_spacings(table:pGtkTable; spacing:gint); cdecl;

  Procedure gtk_table_set_homogeneous(table:pGtkTable; homogeneous:gint); cdecl;


  implementation

const External_library='gtk'; {Setup as you need!}

  function homogeneous(var a : TGtkTable) : guint;
    begin
       homogeneous:=(a.flag0 and bm_homogeneous) shr bp_homogeneous;
    end;

  procedure set_homogeneous(var a : TGtkTable; __homogeneous : guint);
    begin
       a.flag0:=a.flag0 or ((__homogeneous shl bp_homogeneous) and bm_homogeneous);
    end;

  function xexpand(var a : TGtkTableChild) : guint;
    begin
       xexpand:=(a.flag0 and bm_xexpand) shr bp_xexpand;
    end;

  procedure set_xexpand(var a : TGtkTableChild; __xexpand : guint);
    begin
       a.flag0:=a.flag0 or ((__xexpand shl bp_xexpand) and bm_xexpand);
    end;

  function yexpand(var a : TGtkTableChild) : guint;
    begin
       yexpand:=(a.flag0 and bm_yexpand) shr bp_yexpand;
    end;

  procedure set_yexpand(var a : TGtkTableChild; __yexpand : guint);
    begin
       a.flag0:=a.flag0 or ((__yexpand shl bp_yexpand) and bm_yexpand);
    end;

  function xshrink(var a : TGtkTableChild) : guint;
    begin
       xshrink:=(a.flag0 and bm_xshrink) shr bp_xshrink;
    end;

  procedure set_xshrink(var a : TGtkTableChild; __xshrink : guint);
    begin
       a.flag0:=a.flag0 or ((__xshrink shl bp_xshrink) and bm_xshrink);
    end;

  function yshrink(var a : TGtkTableChild) : guint;
    begin
       yshrink:=(a.flag0 and bm_yshrink) shr bp_yshrink;
    end;

  procedure set_yshrink(var a : TGtkTableChild; __yshrink : guint);
    begin
       a.flag0:=a.flag0 or ((__yshrink shl bp_yshrink) and bm_yshrink);
    end;

  function xfill(var a : TGtkTableChild) : guint;
    begin
       xfill:=(a.flag0 and bm_xfill) shr bp_xfill;
    end;

  procedure set_xfill(var a : TGtkTableChild; __xfill : guint);
    begin
       a.flag0:=a.flag0 or ((__xfill shl bp_xfill) and bm_xfill);
    end;

  function yfill(var a : TGtkTableChild) : guint;
    begin
       yfill:=(a.flag0 and bm_yfill) shr bp_yfill;
    end;

  procedure set_yfill(var a : TGtkTableChild; __yfill : guint);
    begin
       a.flag0:=a.flag0 or ((__yfill shl bp_yfill) and bm_yfill);
    end;

  function need_expand(var a : TGtkTableRowCol) : guint;
    begin
       need_expand:=(a.flag0 and bm_need_expand) shr bp_need_expand;
    end;

  procedure set_need_expand(var a : TGtkTableRowCol; __need_expand : guint);
    begin
       a.flag0:=a.flag0 or ((__need_expand shl bp_need_expand) and bm_need_expand);
    end;

  function need_shrink(var a : TGtkTableRowCol) : guint;
    begin
       need_shrink:=(a.flag0 and bm_need_shrink) shr bp_need_shrink;
    end;

  procedure set_need_shrink(var a : TGtkTableRowCol; __need_shrink : guint);
    begin
       a.flag0:=a.flag0 or ((__need_shrink shl bp_need_shrink) and bm_need_shrink);
    end;

  function expand(var a : TGtkTableRowCol) : guint;
    begin
       expand:=(a.flag0 and bm_expand) shr bp_expand;
    end;

  procedure set_expand(var a : TGtkTableRowCol; __expand : guint);
    begin
       a.flag0:=a.flag0 or ((__expand shl bp_expand) and bm_expand);
    end;

  function shrink(var a : TGtkTableRowCol) : guint;
    begin
       shrink:=(a.flag0 and bm_shrink) shr bp_shrink;
    end;

  procedure set_shrink(var a : TGtkTableRowCol; __shrink : guint);
    begin
       a.flag0:=a.flag0 or ((__shrink shl bp_shrink) and bm_shrink);
    end;

  function gtk_table_get_type:guint;CDECL; external;

  function gtk_table_new(rows:gint; columns:gint; homogeneous:gint):PGtkWidget;CDECL; external;

  procedure gtk_table_attach(table:pGtkTable; child:pGtkWidget; left_attach:gint; right_attach:gint; top_attach:gint; 
              bottom_attach:gint; xoptions:gint; yoptions:gint; xpadding:gint; ypadding:gint);CDECL; external;

  procedure gtk_table_attach_defaults(table:pGtkTable; widget:pGtkWidget; left_attach:gint; right_attach:gint; top_attach:gint; 
              bottom_attach:gint);CDECL; external;

  procedure gtk_table_set_row_spacing(table:pGtkTable; row:gint; spacing:gint);CDECL; external;

  procedure gtk_table_set_col_spacing(table:pGtkTable; column:gint; spacing:gint);CDECL; external;

  procedure gtk_table_set_row_spacings(table:pGtkTable; spacing:gint);CDECL; external;

  procedure gtk_table_set_col_spacings(table:pGtkTable; spacing:gint);CDECL; external;

  procedure gtk_table_set_homogeneous(table:pGtkTable; homogeneous:gint);CDECL; external;


end.
